@@ -26,6 +26,12 @@
 #ifndef ACB_POLY_H
 #define ACB_POLY_H
 
+#ifdef ACB_POLY_INLINES_C
+#define ACB_POLY_INLINE
+#else
+#define ACB_POLY_INLINE static __inline__
+#endif
+
 #include "acb.h"
 #include "arb_poly.h"
 
@@ -58,7 +64,7 @@ void _acb_poly_set_length(acb_poly_t poly, long len);
 
 void _acb_poly_normalise(acb_poly_t poly);
 
-static __inline__ void
+ACB_POLY_INLINE void
 acb_poly_swap(acb_poly_t poly1, acb_poly_t poly2)
 {
     acb_poly_struct t = *poly1;
@@ -66,22 +72,22 @@ acb_poly_swap(acb_poly_t poly1, acb_poly_t poly2)
     *poly2 = t;
 }
 
-static __inline__ long acb_poly_length(const acb_poly_t poly)
+ACB_POLY_INLINE long acb_poly_length(const acb_poly_t poly)
 {
     return poly->length;
 }
 
-static __inline__ long acb_poly_degree(const acb_poly_t poly)
+ACB_POLY_INLINE long acb_poly_degree(const acb_poly_t poly)
 {
     return poly->length - 1;
 }
 
-static __inline__ void acb_poly_zero(acb_poly_t poly)
+ACB_POLY_INLINE void acb_poly_zero(acb_poly_t poly)
 {
     poly->length = 0;
 }
 
-static __inline__ void
+ACB_POLY_INLINE void
 acb_poly_one(acb_poly_t poly)
 {
     acb_poly_fit_length(poly, 1);
@@ -106,7 +112,7 @@ void _acb_poly_shift_left(acb_ptr res, acb_srcptr poly, long len, long n);
 
 void acb_poly_shift_left(acb_poly_t res, const acb_poly_t poly, long n);
 
-static __inline__ void
+ACB_POLY_INLINE void
 acb_poly_truncate(acb_poly_t poly, long newlen)
 {
     if (poly->length > newlen)
@@ -149,6 +155,8 @@ void acb_poly_integral(acb_poly_t res, const acb_poly_t poly, long prec);
 
 void acb_poly_set(acb_poly_t dest, const acb_poly_t src);
 
+void acb_poly_set_round(acb_poly_t dest, const acb_poly_t src, long prec);
+
 void acb_poly_set_arb_poly(acb_poly_t poly, const arb_poly_t re);
 
 void acb_poly_set2_arb_poly(acb_poly_t poly, const arb_poly_t re, const arb_poly_t im);
@@ -159,7 +167,11 @@ void acb_poly_set2_fmpq_poly(acb_poly_t poly, const fmpq_poly_t re, const fmpq_p
 
 void acb_poly_set_fmpz_poly(acb_poly_t poly, const fmpz_poly_t src, long prec);
 
-static __inline__ void
+void acb_poly_set2_fmpz_poly(acb_poly_t poly, const fmpz_poly_t re, const fmpz_poly_t im, long prec);
+
+int acb_poly_get_unique_fmpz_poly(fmpz_poly_t res, const acb_poly_t src);
+
+ACB_POLY_INLINE void
 acb_poly_set_acb(acb_poly_t poly, const acb_t c)
 {
     acb_poly_fit_length(poly, 1);
@@ -184,6 +196,12 @@ int acb_poly_overlaps(const acb_poly_t poly1, const acb_poly_t poly2);
 
 int acb_poly_contains(const acb_poly_t poly1, const acb_poly_t poly2);
 
+ACB_POLY_INLINE int
+acb_poly_is_real(acb_poly_t poly)
+{
+    return _acb_vec_is_real(poly->coeffs, poly->length);
+}
+
 void _acb_poly_add(acb_ptr res, acb_srcptr poly1, long len1,
     acb_srcptr poly2, long len2, long prec);
 
@@ -196,7 +214,7 @@ void _acb_poly_sub(acb_ptr res, acb_srcptr poly1, long len1,
 void acb_poly_sub(acb_poly_t res, const acb_poly_t poly1,
               const acb_poly_t poly2, long prec);
 
-static __inline__ void
+ACB_POLY_INLINE void
 acb_poly_neg(acb_poly_t res, const acb_poly_t poly)
 {
     acb_poly_fit_length(res, poly->length);
@@ -204,7 +222,7 @@ acb_poly_neg(acb_poly_t res, const acb_poly_t poly)
     _acb_poly_set_length(res, poly->length);
 }
 
-static __inline__ void
+ACB_POLY_INLINE void
 acb_poly_scalar_mul_2exp_si(acb_poly_t res, const acb_poly_t poly, long c)
 {
     acb_poly_fit_length(res, poly->length);
@@ -251,7 +269,7 @@ void _acb_poly_mul(acb_ptr C,
 void acb_poly_mul(acb_poly_t res, const acb_poly_t poly1,
               const acb_poly_t poly2, long prec);
 
-static __inline__ void
+ACB_POLY_INLINE void
 _acb_poly_mul_monic(acb_ptr res, acb_srcptr poly1, long len1,
     acb_srcptr poly2, long len2, long prec)
 {
@@ -283,7 +301,7 @@ void _acb_poly_rem(acb_ptr R,
     acb_srcptr A, long lenA,
     acb_srcptr B, long lenB, long prec);
 
-void acb_poly_divrem(acb_poly_t Q, acb_poly_t R,
+int acb_poly_divrem(acb_poly_t Q, acb_poly_t R,
                              const acb_poly_t A, const acb_poly_t B, long prec);
 
 void _acb_poly_div_root(acb_ptr Q, acb_t R, acb_srcptr A,
@@ -435,6 +453,16 @@ long acb_poly_find_roots(acb_ptr roots,
 
 /* Special functions */
 
+void _acb_poly_pow_ui_trunc_binexp(acb_ptr res,
+    acb_srcptr f, long flen, ulong exp, long len, long prec);
+
+void acb_poly_pow_ui_trunc_binexp(acb_poly_t res,
+    const acb_poly_t poly, ulong exp, long len, long prec);
+
+void _acb_poly_pow_ui(acb_ptr res, acb_srcptr f, long flen, ulong exp, long prec);
+
+void acb_poly_pow_ui(acb_poly_t res, const acb_poly_t poly, ulong exp, long prec);
+
 void _acb_poly_rsqrt_series(acb_ptr g, acb_srcptr h, long hlen, long len, long prec);
 
 void acb_poly_rsqrt_series(acb_poly_t g, const acb_poly_t h, long n, long prec);
@@ -506,8 +534,25 @@ void _acb_poly_rising_ui_series(acb_ptr res, acb_srcptr f, long flen, ulong r, l
 
 void acb_poly_rising_ui_series(acb_poly_t res, const acb_poly_t f, ulong r, long trunc, long prec);
 
+void _acb_poly_pow_acb_series(acb_ptr h,
+    acb_srcptr f, long flen, const acb_t g, long len, long prec);
+
+void acb_poly_pow_acb_series(acb_poly_t h,
+    const acb_poly_t f, const acb_t g, long len, long prec);
+
+void _acb_poly_pow_series(acb_ptr h,
+    acb_srcptr f, long flen,
+    acb_srcptr g, long glen, long len, long prec);
+
+void acb_poly_pow_series(acb_poly_t h,
+    const acb_poly_t f, const acb_poly_t g, long len, long prec);
+
+void
+_acb_poly_binomial_pow_acb_series(acb_ptr h, acb_srcptr f, long flen,
+    const acb_t g, long len, long prec);
+
 /* TODO: document */
-static __inline__ void
+ACB_POLY_INLINE void
 _acb_poly_acb_pow_cpx(acb_ptr w, const acb_t a, const acb_t b, long len, long prec)
 {
     if (len == 1)
@@ -567,7 +612,18 @@ void _acb_poly_polylog_cpx(acb_ptr w, const acb_t s, const acb_t z, long len, lo
 void _acb_poly_polylog_series(acb_ptr res, acb_srcptr s, long slen, const acb_t z, long len, long prec);
 void acb_poly_polylog_series(acb_poly_t res, const acb_poly_t s, const acb_t z, long n, long prec);
 
+void _acb_poly_agm1_series(acb_ptr res, acb_srcptr z, long zlen, long len, long prec);
+void acb_poly_agm1_series(acb_poly_t res, const acb_poly_t z, long n, long prec);
+void _acb_poly_elliptic_k_series(acb_ptr res, acb_srcptr z, long zlen, long len, long prec);
+void acb_poly_elliptic_k_series(acb_poly_t res, const acb_poly_t z, long n, long prec);
+void _acb_poly_elliptic_p_series(acb_ptr res, acb_srcptr z, long zlen, const acb_t tau, long len, long prec);
+void acb_poly_elliptic_p_series(acb_poly_t res, const acb_poly_t z, const acb_t tau, long n, long prec);
 
+void _acb_poly_erf_series(acb_ptr g, acb_srcptr h, long hlen, long n, long prec);
+void acb_poly_erf_series(acb_poly_t g, const acb_poly_t h, long n, long prec);
+
+void _acb_poly_gamma_upper_series(acb_ptr g, const acb_t s, acb_srcptr h, long hlen, long n, long prec);
+void acb_poly_gamma_upper_series(acb_poly_t g, const acb_t s, const acb_poly_t h, long n, long prec);
 
 #ifdef __cplusplus
 }
