@@ -26,6 +26,12 @@
 #ifndef ARB_H
 #define ARB_H
 
+#ifdef ARB_INLINES_C
+#define ARB_INLINE
+#else
+#define ARB_INLINE static __inline__
+#endif
+
 #include "fmprb.h"
 #include "mag.h"
 #include "arf.h"
@@ -34,6 +40,8 @@
 extern "C" {
 #endif
 
+extern const char * arb_version;
+
 typedef struct
 {
     arf_struct mid;
@@ -52,21 +60,21 @@ typedef const arb_struct * arb_srcptr;
 
 #define ARB_RND ARF_RND_DOWN
 
-static __inline__ void
+ARB_INLINE void
 arb_init(arb_t x)
 {
     arf_init(arb_midref(x));
     mag_init(arb_radref(x));
 }
 
-static __inline__ void
+ARB_INLINE void
 arb_clear(arb_t x)
 {
     arf_clear(arb_midref(x));
     mag_clear(arb_radref(x));
 }
 
-static __inline__ arb_ptr
+ARB_INLINE arb_ptr
 _arb_vec_init(long n)
 {
     long i;
@@ -78,7 +86,7 @@ _arb_vec_init(long n)
     return v;
 }
 
-static __inline__ void
+ARB_INLINE void
 _arb_vec_clear(arb_ptr v, long n)
 {
     long i;
@@ -87,75 +95,75 @@ _arb_vec_clear(arb_ptr v, long n)
     flint_free(v);
 }
 
-static __inline__ void
+ARB_INLINE void
 arb_set_fmprb(arb_t x, const fmprb_t y)
 {
     arf_set_fmpr(arb_midref(x), fmprb_midref(y));
     mag_set_fmpr(arb_radref(x), fmprb_radref(y));
 }
 
-static __inline__ void
+ARB_INLINE void
 arb_get_fmprb(fmprb_t x, const arb_t y)
 {
     arf_get_fmpr(fmprb_midref(x), arb_midref(y));
     mag_get_fmpr(fmprb_radref(x), arb_radref(y));
 }
 
-static __inline__ int
+ARB_INLINE int
 arb_is_exact(const arb_t x)
 {
     return mag_is_zero(arb_radref(x));
 }
 
-static __inline__ int
+ARB_INLINE int
 arb_equal(const arb_t x, const arb_t y)
 {
     return arf_equal(arb_midref(x), arb_midref(y)) &&
            mag_equal(arb_radref(x), arb_radref(y));
 }
 
-static __inline__ void
+ARB_INLINE void
 arb_zero(arb_t x)
 {
     arf_zero(arb_midref(x));
     mag_zero(arb_radref(x));
 }
 
-static __inline__ int
+ARB_INLINE int
 arb_is_zero(const arb_t x)
 {
     return arf_is_zero(arb_midref(x)) && mag_is_zero(arb_radref(x));
 }
 
-static __inline__ void
+ARB_INLINE void
 arb_pos_inf(arb_t x)
 {
     arf_pos_inf(arb_midref(x));
     mag_zero(arb_radref(x));
 }
 
-static __inline__ void
+ARB_INLINE void
 arb_neg_inf(arb_t x)
 {
     arf_neg_inf(arb_midref(x));
     mag_zero(arb_radref(x));
 }
 
-static __inline__ void
+ARB_INLINE void
 arb_zero_pm_inf(arb_t x)
 {
     arf_zero(arb_midref(x));
     mag_inf(arb_radref(x));
 }
 
-static __inline__ void
+ARB_INLINE void
 arb_indeterminate(arb_t x)
 {
     arf_nan(arb_midref(x));
     mag_inf(arb_radref(x));
 }
 
-static __inline__ int
+ARB_INLINE int
 arb_is_finite(const arb_t x)
 {
     return arf_is_finite(arb_midref(x)) && mag_is_finite(arb_radref(x));
@@ -163,7 +171,7 @@ arb_is_finite(const arb_t x)
 
 void arb_set(arb_t x, const arb_t y);
 
-static __inline__ void
+ARB_INLINE void
 arb_swap(arb_t x, arb_t y)
 {
     arb_struct t = *x;
@@ -181,35 +189,46 @@ void arb_neg_round(arb_t x, const arb_t y, long prec);
 
 void arb_abs(arb_t x, const arb_t y);
 
-static __inline__ void
+void _arb_digits_round_inplace(char * s, mp_bitcnt_t * shift, fmpz_t error, long n, arf_rnd_t rnd);
+
+int arb_set_str(arb_t res, const char * inp, long prec);
+
+#define ARB_STR_MORE 1UL
+#define ARB_STR_NO_RADIUS 2UL
+#define ARB_STR_CONDENSE 16UL
+
+char * arb_get_str(const arb_t x, long n, ulong flags);
+
+
+ARB_INLINE void
 arb_set_arf(arb_t x, const arf_t y)
 {
     arf_set(arb_midref(x), y);
     mag_zero(arb_radref(x));
 }
 
-static __inline__ void
+ARB_INLINE void
 arb_set_si(arb_t x, long y)
 {
     arf_set_si(arb_midref(x), y);
     mag_zero(arb_radref(x));
 }
 
-static __inline__ void
+ARB_INLINE void
 arb_set_ui(arb_t x, ulong y)
 {
     arf_set_ui(arb_midref(x), y);
     mag_zero(arb_radref(x));
 }
 
-static __inline__ void
+ARB_INLINE void
 arb_set_fmpz(arb_t x, const fmpz_t y)
 {
     arf_set_fmpz(arb_midref(x), y);
     mag_zero(arb_radref(x));
 }
 
-static __inline__ void
+ARB_INLINE void
 arb_set_fmpz_2exp(arb_t x, const fmpz_t y, const fmpz_t exp)
 {
     arf_set_fmpz_2exp(arb_midref(x), y, exp);
@@ -220,68 +239,58 @@ void arb_set_round_fmpz_2exp(arb_t y, const fmpz_t x, const fmpz_t exp, long pre
 
 void arb_set_round_fmpz(arb_t y, const fmpz_t x, long prec);
 
-static __inline__ int
+ARB_INLINE int
 arb_is_one(const arb_t f)
 {
     return arf_is_one(arb_midref(f)) && mag_is_zero(arb_radref(f));
 }
 
-static __inline__ void
+ARB_INLINE void
 arb_one(arb_t f)
 {
     arb_set_ui(f, 1UL);
 }
 
-static __inline__ void
-arb_print(const arb_t x)
-{
-    arf_print(arb_midref(x));
-    printf(" +/- ");
-    mag_print(arb_radref(x));
-}
+void arb_print(const arb_t x);
 
-static __inline__ void
-arb_printd(const arb_t x, long digits)
-{
-    arf_printd(arb_midref(x), FLINT_MAX(digits, 1));
-    printf(" +/- ");
-    mag_printd(arb_radref(x), 5);
-}
+void arb_printd(const arb_t x, long digits);
 
-static __inline__ void
+void arb_printn(const arb_t x, long digits, ulong flags);
+
+ARB_INLINE void
 arb_mul_2exp_si(arb_t y, const arb_t x, long e)
 {
     arf_mul_2exp_si(arb_midref(y), arb_midref(x), e);
     mag_mul_2exp_si(arb_radref(y), arb_radref(x), e);
 }
 
-static __inline__ void
+ARB_INLINE void
 arb_mul_2exp_fmpz(arb_t y, const arb_t x, const fmpz_t e)
 {
     arf_mul_2exp_fmpz(arb_midref(y), arb_midref(x), e);
     mag_mul_2exp_fmpz(arb_radref(y), arb_radref(x), e);
 }
 
-static __inline__ int
+ARB_INLINE int
 arb_is_int(const arb_t x)
 {
     return mag_is_zero(arb_radref(x)) &&
            arf_is_int(arb_midref(x));
 }
 
-static __inline__ int
+ARB_INLINE int
 arb_contains_zero(const arb_t x)
 {
     return arf_cmpabs_mag(arb_midref(x), arb_radref(x)) <= 0;
 }
 
-static __inline__ int
+ARB_INLINE int
 arb_is_nonzero(const arb_t x)
 {
     return !arb_contains_zero(x);
 }
 
-static __inline__ int
+ARB_INLINE int
 arb_is_positive(const arb_t x)
 {
     return (arf_sgn(arb_midref(x)) > 0) &&
@@ -289,7 +298,7 @@ arb_is_positive(const arb_t x)
          !arf_is_nan(arb_midref(x));
 }
 
-static __inline__ int
+ARB_INLINE int
 arb_is_nonnegative(const arb_t x)
 {
     return (arf_sgn(arb_midref(x)) >= 0) &&
@@ -297,7 +306,7 @@ arb_is_nonnegative(const arb_t x)
          !arf_is_nan(arb_midref(x));
 }
 
-static __inline__ int
+ARB_INLINE int
 arb_is_negative(const arb_t x)
 {
     return (arf_sgn(arb_midref(x)) < 0) &&
@@ -305,7 +314,7 @@ arb_is_negative(const arb_t x)
          !arf_is_nan(arb_midref(x));
 }
 
-static __inline__ int
+ARB_INLINE int
 arb_is_nonpositive(const arb_t x)
 {
     return (arf_sgn(arb_midref(x)) <= 0) &&
@@ -313,7 +322,7 @@ arb_is_nonpositive(const arb_t x)
          !arf_is_nan(arb_midref(x));
 }
 
-static __inline__ int
+ARB_INLINE int
 arb_contains_negative(const arb_t x)
 {
     return (arf_sgn(arb_midref(x)) < 0) ||
@@ -321,7 +330,7 @@ arb_contains_negative(const arb_t x)
         || arf_is_nan(arb_midref(x));
 }
 
-static __inline__ int
+ARB_INLINE int
 arb_contains_nonpositive(const arb_t x)
 {
     return (arf_sgn(arb_midref(x)) <= 0) ||
@@ -329,7 +338,7 @@ arb_contains_nonpositive(const arb_t x)
         || arf_is_nan(arb_midref(x));
 }
 
-static __inline__ int
+ARB_INLINE int
 arb_contains_positive(const arb_t x)
 {
     return (arf_sgn(arb_midref(x)) > 0) ||
@@ -337,7 +346,7 @@ arb_contains_positive(const arb_t x)
         || arf_is_nan(arb_midref(x));
 }
 
-static __inline__ int
+ARB_INLINE int
 arb_contains_nonnegative(const arb_t x)
 {
     return (arf_sgn(arb_midref(x)) >= 0) ||
@@ -347,7 +356,9 @@ arb_contains_nonnegative(const arb_t x)
 
 void arb_get_mag_lower(mag_t z, const arb_t x);
 
-static __inline__ void
+void arb_get_mag_lower_nonnegative(mag_t z, const arb_t x);
+
+ARB_INLINE void
 arb_get_mag(mag_t z, const arb_t x)
 {
     mag_t t;
@@ -356,7 +367,7 @@ arb_get_mag(mag_t z, const arb_t x)
     mag_clear(t);
 }
 
-static __inline__ void
+ARB_INLINE void
 arb_get_abs_ubound_arf(arf_t u, const arb_t x, long prec)
 {
     arf_t t;
@@ -370,7 +381,7 @@ arb_get_abs_ubound_arf(arf_t u, const arb_t x, long prec)
     arf_abs(u, u);
 }
 
-static __inline__ void
+ARB_INLINE void
 arb_get_abs_lbound_arf(arf_t u, const arb_t x, long prec)
 {
     arf_t t;
@@ -392,13 +403,13 @@ arb_get_abs_lbound_arf(arf_t u, const arb_t x, long prec)
 
 long arb_rel_error_bits(const arb_t x);
 
-static __inline__ long
+ARB_INLINE long
 arb_rel_accuracy_bits(const arb_t x)
 {
     return -arb_rel_error_bits(x);
 }
 
-static __inline__ long
+ARB_INLINE long
 arb_bits(const arb_t x)
 {
     return arf_bits(arb_midref(x));
@@ -423,7 +434,7 @@ void arb_add_error_2exp_fmpz(arb_t x, const fmpz_t err);
 void arb_add_error(arb_t x, const arb_t error);
 
 /* TODO: document */
-static __inline__ void
+ARB_INLINE void
 arb_add_error_mag(arb_t x, const mag_t err)
 {
     mag_add(arb_radref(x), arb_radref(x), err);
@@ -446,6 +457,11 @@ int arb_contains(const arb_t x, const arb_t y);
 void arb_get_interval_fmpz_2exp(fmpz_t a, fmpz_t b, fmpz_t exp, const arb_t x);
 int arb_get_unique_fmpz(fmpz_t z, const arb_t x);
 
+void arb_get_fmpz_mid_rad_10exp(fmpz_t mid, fmpz_t rad, fmpz_t exp, const arb_t x, long n);
+
+void arb_floor(arb_t z, const arb_t x, long prec);
+void arb_ceil(arb_t z, const arb_t x, long prec);
+
 void arb_set_interval_arf(arb_t x, const arf_t a, const arf_t b, long prec);
 void arb_set_interval_mpfr(arb_t x, const mpfr_t a, const mpfr_t b, long prec);
 
@@ -494,13 +510,13 @@ void arb_div_fmpz(arb_t z, const arb_t x, const fmpz_t y, long prec);
 void arb_fmpz_div_fmpz(arb_t z, const fmpz_t x, const fmpz_t y, long prec);
 void arb_ui_div(arb_t z, ulong x, const arb_t y, long prec);
 
-static __inline__ void
+ARB_INLINE void
 arb_inv(arb_t y, const arb_t x, long prec)
 {
     arb_ui_div(y, 1, x, prec);
 }
 
-static __inline__ void
+ARB_INLINE void
 arb_set_fmpq(arb_t y, const fmpq_t x, long prec)
 {
     arb_fmpz_div_fmpz(y, fmpq_numref(x), fmpq_denref(x), prec);
@@ -516,6 +532,7 @@ void arb_hypot(arb_t z, const arb_t x, const arb_t y, long prec);
 
 void arb_rsqrt(arb_t z, const arb_t x, long prec);
 void arb_rsqrt_ui(arb_t z, ulong x, long prec);
+void arb_sqrt1pm1(arb_t r, const arb_t z, long prec);
 
 void arb_pow_fmpz_binexp(arb_t y, const arb_t b, const fmpz_t e, long prec);
 void arb_pow_fmpz(arb_t y, const arb_t b, const fmpz_t e, long prec);
@@ -531,6 +548,7 @@ void arb_log(arb_t z, const arb_t x, long prec);
 void arb_log_arf(arb_t z, const arf_t x, long prec);
 void arb_log_ui(arb_t z, ulong x, long prec);
 void arb_log_fmpz(arb_t z, const fmpz_t x, long prec);
+void arb_log1p(arb_t r, const arb_t z, long prec);
 void arb_exp(arb_t z, const arb_t x, long prec);
 void arb_expm1(arb_t z, const arb_t x, long prec);
 void arb_sin(arb_t s, const arb_t x, long prec);
@@ -559,6 +577,9 @@ void arb_atan(arb_t z, const arb_t x, long prec);
 void arb_atan2(arb_t z, const arb_t b, const arb_t a, long prec);
 void arb_asin(arb_t z, const arb_t x, long prec);
 void arb_acos(arb_t z, const arb_t x, long prec);
+void arb_atanh(arb_t z, const arb_t x, long prec);
+void arb_asinh(arb_t z, const arb_t x, long prec);
+void arb_acosh(arb_t z, const arb_t x, long prec);
 void arb_fac_ui(arb_t z, ulong n, long prec);
 void arb_bin_ui(arb_t z, const arb_t n, ulong k, long prec);
 void arb_bin_uiui(arb_t z, ulong n, ulong k, long prec);
@@ -583,6 +604,7 @@ void arb_gamma_fmpz(arb_t z, const fmpz_t x, long prec);
 void arb_digamma(arb_t y, const arb_t x, long prec);
 void arb_zeta(arb_t z, const arb_t s, long prec);
 void arb_zeta_ui(arb_t z, ulong n, long prec);
+void arb_hurwitz_zeta(arb_t z, const arb_t s, const arb_t a, long prec);
 void arb_bernoulli_ui(arb_t z, ulong n, long prec);
 
 void arb_rising_ui_bs(arb_t y, const arb_t x, ulong n, long prec);
@@ -614,6 +636,11 @@ void arb_bernoulli_ui_zeta(arb_t b, ulong n, long prec);
 void arb_polylog(arb_t w, const arb_t s, const arb_t z, long prec);
 void arb_polylog_si(arb_t w, long s, const arb_t z, long prec);
 
+void arb_chebyshev_t_ui(arb_t a, ulong n, const arb_t x, long prec);
+void arb_chebyshev_t2_ui(arb_t a, arb_t b, ulong n, const arb_t x, long prec);
+void arb_chebyshev_u_ui(arb_t a, ulong n, const arb_t x, long prec);
+void arb_chebyshev_u2_ui(arb_t a, arb_t b, ulong n, const arb_t x, long prec);
+
 #define ARB_DEF_CACHED_CONSTANT(name, comp_func) \
     TLS_PREFIX long name ## _cached_prec = 0; \
     TLS_PREFIX arb_t name ## _cached_value; \
@@ -639,7 +666,7 @@ void arb_polylog_si(arb_t w, long s, const arb_t z, long prec);
 
 /* vector functions */
 
-static __inline__ void
+ARB_INLINE void
 _arb_vec_zero(arb_ptr A, long n)
 {
     long i;
@@ -647,7 +674,7 @@ _arb_vec_zero(arb_ptr A, long n)
         arb_zero(A + i);
 }
 
-static __inline__ int
+ARB_INLINE int
 _arb_vec_is_zero(arb_srcptr vec, long len)
 {
     long i;
@@ -657,7 +684,7 @@ _arb_vec_is_zero(arb_srcptr vec, long len)
     return 1;
 }
 
-static __inline__ int
+ARB_INLINE int
 _arb_vec_is_finite(arb_srcptr x, long len)
 {
     long i;
@@ -669,7 +696,7 @@ _arb_vec_is_finite(arb_srcptr x, long len)
     return 1;
 }
 
-static __inline__ void
+ARB_INLINE void
 _arb_vec_set(arb_ptr res, arb_srcptr vec, long len)
 {
     long i;
@@ -677,7 +704,7 @@ _arb_vec_set(arb_ptr res, arb_srcptr vec, long len)
         arb_set(res + i, vec + i);
 }
 
-static __inline__ void
+ARB_INLINE void
 _arb_vec_set_round(arb_ptr res, arb_srcptr vec, long len, long prec)
 {
     long i;
@@ -685,7 +712,7 @@ _arb_vec_set_round(arb_ptr res, arb_srcptr vec, long len, long prec)
         arb_set_round(res + i, vec + i, prec);
 }
 
-static __inline__ void
+ARB_INLINE void
 _arb_vec_swap(arb_ptr res, arb_ptr vec, long len)
 {
     long i;
@@ -693,7 +720,7 @@ _arb_vec_swap(arb_ptr res, arb_ptr vec, long len)
         arb_swap(res + i, vec + i);
 }
 
-static __inline__ void
+ARB_INLINE void
 _arb_vec_neg(arb_ptr B, arb_srcptr A, long n)
 {
     long i;
@@ -701,7 +728,7 @@ _arb_vec_neg(arb_ptr B, arb_srcptr A, long n)
         arb_neg(B + i, A + i);
 }
 
-static __inline__ void
+ARB_INLINE void
 _arb_vec_sub(arb_ptr C, arb_srcptr A,
     arb_srcptr B, long n, long prec)
 {
@@ -710,7 +737,7 @@ _arb_vec_sub(arb_ptr C, arb_srcptr A,
         arb_sub(C + i, A + i, B + i, prec);
 }
 
-static __inline__ void
+ARB_INLINE void
 _arb_vec_add(arb_ptr C, arb_srcptr A,
     arb_srcptr B, long n, long prec)
 {
@@ -719,7 +746,7 @@ _arb_vec_add(arb_ptr C, arb_srcptr A,
         arb_add(C + i, A + i, B + i, prec);
 }
 
-static __inline__ void
+ARB_INLINE void
 _arb_vec_scalar_mul(arb_ptr res, arb_srcptr vec,
     long len, const arb_t c, long prec)
 {
@@ -728,7 +755,7 @@ _arb_vec_scalar_mul(arb_ptr res, arb_srcptr vec,
         arb_mul(res + i, vec + i, c, prec);
 }
 
-static __inline__ void
+ARB_INLINE void
 _arb_vec_scalar_div(arb_ptr res, arb_srcptr vec,
     long len, const arb_t c, long prec)
 {
@@ -737,7 +764,7 @@ _arb_vec_scalar_div(arb_ptr res, arb_srcptr vec,
         arb_div(res + i, vec + i, c, prec);
 }
 
-static __inline__ void
+ARB_INLINE void
 _arb_vec_scalar_mul_fmpz(arb_ptr res, arb_srcptr vec,
     long len, const fmpz_t c, long prec)
 {
@@ -750,7 +777,7 @@ _arb_vec_scalar_mul_fmpz(arb_ptr res, arb_srcptr vec,
     arf_clear(t);
 }
 
-static __inline__ void
+ARB_INLINE void
 _arb_vec_scalar_mul_2exp_si(arb_ptr res, arb_srcptr src, long len, long c)
 {
     long i;
@@ -758,7 +785,7 @@ _arb_vec_scalar_mul_2exp_si(arb_ptr res, arb_srcptr src, long len, long c)
         arb_mul_2exp_si(res + i, src + i, c);
 }
 
-static __inline__ void
+ARB_INLINE void
 _arb_vec_scalar_addmul(arb_ptr res, arb_srcptr vec,
     long len, const arb_t c, long prec)
 {
@@ -767,7 +794,7 @@ _arb_vec_scalar_addmul(arb_ptr res, arb_srcptr vec,
         arb_addmul(res + i, vec + i, c, prec);
 }
 
-static __inline__ void
+ARB_INLINE void
 _arb_vec_dot(arb_t res, arb_srcptr vec1, arb_srcptr vec2, long len2, long prec)
 {
     long i;
@@ -776,7 +803,7 @@ _arb_vec_dot(arb_t res, arb_srcptr vec1, arb_srcptr vec2, long len2, long prec)
         arb_addmul(res, vec1 + i, vec2 + i, prec);
 }
 
-static __inline__ void
+ARB_INLINE void
 _arb_vec_norm(arb_t res, arb_srcptr vec, long len, long prec)
 {
     long i;
@@ -786,7 +813,7 @@ _arb_vec_norm(arb_t res, arb_srcptr vec, long len, long prec)
 }
 
 /* TODO: mag version? */
-static __inline__ void
+ARB_INLINE void
 _arb_vec_get_abs_ubound_arf(arf_t bound, arb_srcptr vec,
         long len, long prec)
 {
@@ -811,7 +838,7 @@ _arb_vec_get_abs_ubound_arf(arf_t bound, arb_srcptr vec,
     }
 }
 
-static __inline__ long
+ARB_INLINE long
 _arb_vec_bits(arb_srcptr x, long len)
 {
     long i, b, c;
@@ -826,7 +853,7 @@ _arb_vec_bits(arb_srcptr x, long len)
     return b;
 }
 
-static __inline__ void
+ARB_INLINE void
 _arb_vec_set_powers(arb_ptr xs, const arb_t x, long len, long prec)
 {
     long i;
@@ -844,7 +871,7 @@ _arb_vec_set_powers(arb_ptr xs, const arb_t x, long len, long prec)
     }
 }
 
-static __inline__ void
+ARB_INLINE void
 _arb_vec_add_error_arf_vec(arb_ptr res, arf_srcptr err, long len)
 {
     long i;
@@ -852,7 +879,7 @@ _arb_vec_add_error_arf_vec(arb_ptr res, arf_srcptr err, long len)
         arb_add_error_arf(res + i, err + i);
 }
 
-static __inline__ void
+ARB_INLINE void
 _arb_vec_add_error_mag_vec(arb_ptr res, mag_srcptr err, long len)
 {
     long i;
@@ -860,7 +887,7 @@ _arb_vec_add_error_mag_vec(arb_ptr res, mag_srcptr err, long len)
         mag_add(arb_radref(res + i), arb_radref(res + i), err + i);
 }
 
-static __inline__ void
+ARB_INLINE void
 _arb_vec_indeterminate(arb_ptr vec, long len)
 {
     long i;
@@ -868,7 +895,7 @@ _arb_vec_indeterminate(arb_ptr vec, long len)
         arb_indeterminate(vec + i);
 }
 
-static __inline__ void
+ARB_INLINE void
 _arb_vec_trim(arb_ptr res, arb_srcptr vec, long len)
 {
     long i;
@@ -876,6 +903,16 @@ _arb_vec_trim(arb_ptr res, arb_srcptr vec, long len)
         arb_trim(res + i, vec + i);
 }
 
+ARB_INLINE int
+_arb_vec_get_unique_fmpz_vec(fmpz * res,  arb_srcptr vec, long len)
+{
+    long i;
+    for (i = 0; i < len; i++)
+        if (!arb_get_unique_fmpz(res + i, vec + i))
+            return 0;
+    return 1;
+}
+
 /* arctangent implementation */
 
 #define ARB_ATAN_TAB1_BITS 8
@@ -964,12 +1001,12 @@ void _arb_exp_sum_bs_simple(fmpz_t T, fmpz_t Q, mp_bitcnt_t * Qexp,
 #define ARB_SIN_COS_TAB1_PREC 512
 #define ARB_SIN_COS_TAB1_LIMBS (ARB_SIN_COS_TAB1_PREC / FLINT_BITS)
 
-/* only goes up to log(2) * 32 */
+/* only goes up to (pi/4) * 32 */
 #define ARB_SIN_COS_TAB21_NUM 26
 #define ARB_SIN_COS_TAB21_BITS 5
 #define ARB_SIN_COS_TAB22_NUM (1 << ARB_SIN_COS_TAB22_BITS)
 #define ARB_SIN_COS_TAB22_BITS 5
-#define ARB_SIN_COS_TAB2_PREC 2560
+#define ARB_SIN_COS_TAB2_PREC 4608
 #define ARB_SIN_COS_TAB2_LIMBS (ARB_SIN_COS_TAB2_PREC / FLINT_BITS)
 
 extern const mp_limb_t arb_sin_cos_tab1[2 * ARB_SIN_COS_TAB1_NUM][ARB_SIN_COS_TAB1_LIMBS];
@@ -983,11 +1020,46 @@ void _arb_sin_cos_taylor_naive(mp_ptr ysin, mp_ptr ycos, mp_limb_t * error,
     mp_srcptr x, mp_size_t xn, ulong N);
 
 void _arb_sin_cos_taylor_rs(mp_ptr ysin, mp_ptr ycos,
-    mp_limb_t * error, mp_srcptr x, mp_size_t xn, ulong N);
+    mp_limb_t * error, mp_srcptr x, mp_size_t xn, ulong N,
+    int sinonly, int alternating);
 
 int _arb_get_mpn_fixed_mod_pi4(mp_ptr w, fmpz_t q, int * octant,
     mp_limb_t * error, const arf_t x, mp_size_t wn);
 
+ARB_INLINE mp_bitcnt_t
+_arb_mpn_leading_zeros(mp_srcptr d, mp_size_t n)
+{
+    mp_limb_t t;
+    mp_size_t zero_limbs;
+    mp_bitcnt_t bits;
+
+    zero_limbs = 0;
+
+    while (1)
+    {
+        t = d[n - zero_limbs - 1];
+
+        if (t != 0)
+        {
+            count_leading_zeros(bits, t);
+            return bits + FLINT_BITS * zero_limbs;
+        }
+
+        zero_limbs++;
+
+        if (zero_limbs == n)
+            return FLINT_BITS * zero_limbs;
+    }
+}
+
+void _arb_atan_sum_bs_simple(fmpz_t T, fmpz_t Q, mp_bitcnt_t * Qexp,
+    const fmpz_t x, mp_bitcnt_t r, long N);
+
+void _arb_atan_sum_bs_powtab(fmpz_t T, fmpz_t Q, mp_bitcnt_t * Qexp,
+    const fmpz_t x, mp_bitcnt_t r, long N);
+
+void arb_atan_arf_bb(arb_t z, const arf_t x, long prec);
+
 #ifdef __cplusplus
 }
 #endif
