@@ -26,6 +26,12 @@
 #ifndef ARB_POLY_H
 #define ARB_POLY_H
 
+#ifdef ARB_POLY_INLINES_C
+#define ARB_POLY_INLINE
+#else
+#define ARB_POLY_INLINE static __inline__
+#endif
+
 #include "arb.h"
 #include "acb.h"
 #include "fmpz_poly.h"
@@ -60,7 +66,7 @@ void _arb_poly_set_length(arb_poly_t poly, long len);
 
 void _arb_poly_normalise(arb_poly_t poly);
 
-static __inline__ void
+ARB_POLY_INLINE void
 arb_poly_swap(arb_poly_t poly1, arb_poly_t poly2)
 {
     arb_poly_struct t = *poly1;
@@ -70,24 +76,26 @@ arb_poly_swap(arb_poly_t poly1, arb_poly_t poly2)
 
 void arb_poly_set(arb_poly_t poly, const arb_poly_t src);
 
+void arb_poly_set_round(arb_poly_t poly, const arb_poly_t src, long prec);
+
 /* Basic manipulation */
 
-static __inline__ long arb_poly_length(const arb_poly_t poly)
+ARB_POLY_INLINE long arb_poly_length(const arb_poly_t poly)
 {
     return poly->length;
 }
 
-static __inline__ long arb_poly_degree(const arb_poly_t poly)
+ARB_POLY_INLINE long arb_poly_degree(const arb_poly_t poly)
 {
     return poly->length - 1;
 }
 
-static __inline__ void arb_poly_zero(arb_poly_t poly)
+ARB_POLY_INLINE void arb_poly_zero(arb_poly_t poly)
 {
     poly->length = 0;
 }
 
-static __inline__ void
+ARB_POLY_INLINE void
 arb_poly_one(arb_poly_t poly)
 {
     arb_poly_fit_length(poly, 1);
@@ -114,7 +122,7 @@ void _arb_poly_shift_left(arb_ptr res, arb_srcptr poly, long len, long n);
 
 void arb_poly_shift_left(arb_poly_t res, const arb_poly_t poly, long n);
 
-static __inline__ void
+ARB_POLY_INLINE void
 arb_poly_truncate(arb_poly_t poly, long newlen)
 {
     if (poly->length > newlen)
@@ -133,7 +141,7 @@ void arb_poly_set_fmpz_poly(arb_poly_t poly, const fmpz_poly_t src, long prec);
 
 void arb_poly_set_fmpq_poly(arb_poly_t poly, const fmpq_poly_t src, long prec);
 
-static __inline__ void
+ARB_POLY_INLINE void
 arb_poly_set_arb(arb_poly_t poly, const arb_t c)
 {
     arb_poly_fit_length(poly, 1);
@@ -143,6 +151,8 @@ arb_poly_set_arb(arb_poly_t poly, const arb_t c)
 
 void arb_poly_set_si(arb_poly_t poly, long c);
 
+int arb_poly_get_unique_fmpz_poly(fmpz_poly_t res, const arb_poly_t src);
+
 /* Comparisons */
 
 int arb_poly_contains(const arb_poly_t poly1, const arb_poly_t poly2);
@@ -180,7 +190,7 @@ void _arb_poly_sub(arb_ptr res, arb_srcptr poly1, long len1,
 void arb_poly_sub(arb_poly_t res, const arb_poly_t poly1,
               const arb_poly_t poly2, long prec);
 
-static __inline__ void
+ARB_POLY_INLINE void
 arb_poly_neg(arb_poly_t res, const arb_poly_t poly)
 {
     arb_poly_fit_length(res, poly->length);
@@ -188,7 +198,7 @@ arb_poly_neg(arb_poly_t res, const arb_poly_t poly)
     _arb_poly_set_length(res, poly->length);
 }
 
-static __inline__ void
+ARB_POLY_INLINE void
 arb_poly_scalar_mul_2exp_si(arb_poly_t res, const arb_poly_t poly, long c)
 {
     arb_poly_fit_length(res, poly->length);
@@ -233,7 +243,7 @@ void _arb_poly_mul(arb_ptr C,
 void arb_poly_mul(arb_poly_t res, const arb_poly_t poly1,
               const arb_poly_t poly2, long prec);
 
-static __inline__ void
+ARB_POLY_INLINE void
 _arb_poly_mul_monic(arb_ptr res, arb_srcptr poly1, long len1,
     arb_srcptr poly2, long len2, long prec)
 {
@@ -265,7 +275,7 @@ void _arb_poly_rem(arb_ptr R,
     arb_srcptr A, long lenA,
     arb_srcptr B, long lenB, long prec);
 
-void arb_poly_divrem(arb_poly_t Q, arb_poly_t R,
+int arb_poly_divrem(arb_poly_t Q, arb_poly_t R,
                              const arb_poly_t A, const arb_poly_t B, long prec);
 
 void _arb_poly_div_root(arb_ptr Q, arb_t R, arb_srcptr A,
@@ -455,6 +465,8 @@ void _arb_poly_pow_arb_series(arb_ptr h,
 void arb_poly_pow_arb_series(arb_poly_t h,
     const arb_poly_t f, const arb_t g, long len, long prec);
 
+void _arb_poly_binomial_pow_arb_series(arb_ptr h, arb_srcptr f, long flen, const arb_t g, long len, long prec);
+
 void _arb_poly_rsqrt_series(arb_ptr g,
     arb_srcptr h, long hlen, long len, long prec);
 
@@ -567,6 +579,10 @@ void arb_poly_riemann_siegel_theta_series(arb_poly_t res, const arb_poly_t h, lo
 void _arb_poly_riemann_siegel_z_series(arb_ptr res, arb_srcptr h, long hlen, long len, long prec);
 void arb_poly_riemann_siegel_z_series(arb_poly_t res, const arb_poly_t h, long n, long prec);
 
+long _arb_poly_swinnerton_dyer_ui_prec(ulong n);
+void _arb_poly_swinnerton_dyer_ui(arb_ptr T, ulong n, long trunc, long prec);
+void arb_poly_swinnerton_dyer_ui(arb_poly_t poly, ulong n, long prec);
+
 /* Root-finding */
 
 void _arb_poly_newton_convergence_factor(arf_t convergence_factor,
@@ -589,7 +605,7 @@ void _arb_poly_newton_refine_root(arb_t r, arb_srcptr poly,
 
 
 /* counts zero bits in the binary representation of e */
-static __inline__ int
+ARB_POLY_INLINE int
 n_zerobits(mp_limb_t e)
 {
     int zeros = 0;
@@ -603,7 +619,7 @@ n_zerobits(mp_limb_t e)
     return zeros;
 }
 
-static __inline__ long
+ARB_POLY_INLINE long
 poly_pow_length(long poly_len, ulong exp, long trunc)
 {
     mp_limb_t hi, lo;
